import os


def _get_color(color=None):
  """Get color name from pre-defined color list"""
  if color == False or os.environ.get("RAINBOW_LOGGER_NO_COLOR") or color == "":
    return ""

  if color is None and not color:
    raise ValueError("Must have color code")

  colors = {
    "PURPLE":"\033[96m",
    "MAGENTA":"\033[95m",
    "BLUE":"\033[94m",
    "GREEN":"\033[92m",
    "YELLOW":"\033[93m",
    "RED":"\033[91m",
    "DARKGRAY":"\033[90m",
    "GREY":"\033[0m",
    "WHITE":"\033[1m"
  }

  if color not in colors:
    raise KeyError("Use a proper color name: {}".format(list(colors.keys())))

  return colors[color]


def _get_message(no_time=False, no_color=False):
  """Get message parts"""
  time = "{}%(asctime)s{}".format(
    _get_color(not no_color and "DARKGRAY"),
    _get_color(not no_color and "GREY")
  )

  name = "{}%(name)-12s{}".format(
    _get_color(not no_color and "PURPLE"),
    _get_color(not no_color and "GREY")
  )

  level = "%(levelname)-8s"
  message = "%(message)s"

  if os.environ.get("RAINBOW_LOGGER_NO_TIME") == "true" or no_time:
    return "{} {}\t{}".format(name, level, message)

  return "{} {} {}\t{}".format(time, name, level, message)


def _set_level_format(level=None, color="", no_color=False, logging_module=None):
  """Set logging format based on level and color"""
  FORMAT = "{}{}{}"
  LEVEL_MAPPING = {
    10: "DEBUG",
    20: "INFO",
    30: "WARNING",
    40: "ERROR",
    50: "CRITICAL",
  }

  _logging_module = logging_module
  parsed_format = FORMAT.format(
    _get_color(not no_color and color),
    LEVEL_MAPPING[level],
    _get_color(not no_color and "GREY")
  )


  if level is not None:
    _logging_module.addLevelName(level, parsed_format)

  return True



def test_console_named_logger():
  from rainbow import RainbowLogger
  logger = RainbowLogger("rainbow")
  logger.info("my info")
  logger.warning("my warn")
  logger.error("my error")
  logger.debug("my debug")
  logger.critical("my critical")


def test_file_named_logger():
  from rainbow import RainbowLogger
  logger = RainbowLogger("rainbow", filepath="rainbow.named.log")
  logger.info("my info")
  logger.warning("my warn")
  logger.error("my error")
  logger.debug("my debug")
  logger.critical("my critical")


def test_console_root_logger():
  from rainbow import RainbowLogger
  logger = RainbowLogger()
  logger.info("root info")
  logger.warning("root warn")
  logger.error("root error")
  logger.debug("root debug")
  logger.critical("root critical")


def test_file_root_logger():
  from rainbow import RainbowLogger
  logger = RainbowLogger(filepath="rainbow.root.log")
  logger.info("root info")
  logger.warning("root warn")
  logger.error("root error")
  logger.debug("root debug")
  logger.critical("root critical")


if __name__ == "__main__":
  test_console_named_logger()
  test_file_named_logger()

  test_console_root_logger()
  test_file_root_logger()



import os
import sys
from .helpers import _set_level_format
from .helpers import _get_message


class RainbowLogger:
  """A customized logger built on top of Python's logging"""
  def __new__(
    cls,
    name=None,
    no_time=False,
    no_color=False,
    new_logging=None,
    filepath=None,
    log_level=None,
    get_logging=False
  ):
    import logging


    if not log_level:
      log_level = logging.DEBUG

    _logging_module = logging
    if new_logging is not None:
      _logging_module = new_logging

    if filepath:
      no_color = True

    level_color_mapping = [
      {"level": _logging_module.DEBUG, "color": "BLUE"},
      {"level": _logging_module.INFO, "color": "GREEN"},
      {"level": _logging_module.WARN, "color": "YELLOW"},
      {"level": _logging_module.ERROR, "color": "RED"},
      {"level": _logging_module.CRITICAL, "color": "MAGENTA"}
    ]

    for l in level_color_mapping:
      _set_level_format(
        level=l['level'],
        color=l["color"],
        no_color=no_color,
        logging_module=_logging_module
      )

    logger = _logging_module.getLogger(name)
    logger.setLevel(log_level)

    log_level = log_level if log_level else _logging_module.DEBUG
    if 'RAINBOW_LEVEL' in os.environ:
      log_level = getattr(
        _logging_module,
        os.environ["RAINBOW_LEVEL"],
        "DEBUG"
      )

    final_message = ""

    # remove existing handlers
    [logger.removeHandler(h) for h in logger.handlers]

    if not filepath and name:
      handler = _logging_module.StreamHandler(sys.stdout)
      final_message = _get_message(no_time, no_color)
      formatter = _logging_module.Formatter(final_message)
      handler.setFormatter(formatter)
      handler.setLevel(log_level)
      logger.addHandler(handler)
    elif filepath and name:
      # console logs on critical
      handler = _logging_module.StreamHandler()
      final_message = _get_message(no_time, no_color)
      formatter = _logging_module.Formatter(final_message)
      handler.setFormatter(formatter)
      handler.setLevel(_logging_module.CRITICAL)
      logger.addHandler(handler)

      handler = _logging_module.FileHandler(filepath)
      final_message = _get_message(no_time, no_color)
      formatter = _logging_module.Formatter(final_message)
      handler.setFormatter(formatter)
      handler.setLevel(log_level)
      logger.addHandler(handler)

    if not name:
      final_message = _get_message(no_time, no_color)
      config = {
        "format": final_message,
        "level": log_level,
        "filemode": "a",
      }

      if filepath:
        config["filename"] = filepath

      _logging_module.basicConfig(**config)

    if get_logging:
      return _logging_module

    return logger
